/*
 * pitchtune - an instrument tuner
 * Copyright (C) 2000 Haakon Andre Hjortland
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * \file control.c
 * \brief Parameter control routines.
 * 
 * Includes functions that controls the tuner settings, like frequency,
 * note values, fine tune and coarse tune sampepieces, etc...
 */



/*** Includes */

#include <math.h>

#include "main.h"


/**
 * \brief Sets the frequency
 * 
 * \param freq Frequency to set up UI, tuner, and sound playback
 */
void set_frequency(double freq)
{
    gtk_label_set_text(GTK_LABEL(label_freqwarn), " ");

    if (freq < FREQUENCY_MIN)
	{
	    gtk_label_set_text(GTK_LABEL(label_freqwarn), "Frequency too low!!!");
	    freq=FREQUENCY_MIN;
	}
    if (freq > FREQUENCY_MAX)
	{
	    gtk_label_set_text(GTK_LABEL(label_freqwarn), "Frequency too high!!!");
	    freq=FREQUENCY_MAX;
	}

    if (samplerate != -1)
	{
	    frequency_set=TRUE;
	    frequency=freq;
	    wavelength_samples=(double)samplerate / frequency;
	    downsample_ratio=1;

            sound_change_freq(freq);
    
	  label0:
            /* Downsample when less than 3 waveforms */
	    if ((wavelength_samples / downsample_ratio) > (DRAWAREA_WIDTH / 3))
		{
		    downsample_ratio += 1;
		    goto label0;
		}
            /* Upsample when more than 10 waveforms */
	    if ((wavelength_samples / downsample_ratio) < (DRAWAREA_WIDTH / 10))
		{
		    downsample_ratio /= 2;
		    goto label0;
		}
    
	    check_redo_display();
	}

    return;
}



/**
 * \brief Set the fine tuning samplepieces (see \ref Concepts)
 * \param number Number of samplepieces
 */
void set_samplepieces_count_finetune(int number)
{
    if (number < 1)
	{
	    PRINTF_DEBUG("samplepieces_count out of range (%d)\n", number);
	    number=1;
	}

    if (number > SAMPLEPIECES_MAX)
	{
	    PRINTF_DEBUG("samplepieces_count out of range (%d)\n", number);
	    number=SAMPLEPIECES_MAX;
	}

    samplepieces_count_finetune=number;

    check_redo_display();

    return;
}



/**
 * \brief Set the coarse tuning samplepieces (see \ref Concepts)
 * \param number Number of samplepieces
 */

void set_samplepieces_count_coarsetune(int number)
{
    if (number < 1)
	{
	    PRINTF_DEBUG("samplepieces_count out of range (%d)\n", number);
	    number=1;
	}

    if (number > SAMPLEPIECES_MAX)
	{
	    PRINTF_DEBUG("samplepieces_count out of range (%d)\n", number);
	    number=SAMPLEPIECES_MAX;
	}

    samplepieces_count_coarsetune=number;

    check_redo_display();

    return;
}



/** 
 * \brief Set note
 * \param note_set Set the note (?)
 * \param note Note number (?)
 */
void set_note(gboolean note_set, int note)
{
    float float1, float2;

    if (note < NOTE_MIN)
        note = NOTE_MIN;
    if (note > NOTE_MAX)
        note = NOTE_MAX;

    sel_guitar_set_note(note_set, note);

    sel_standard_set_note(note_set, note);

    notes_note = note;
    notes_note_set = note_set;

    if (note_set == TRUE) {
        /* Get ground frequency */
        float1 = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_a1freq));
    
        /*Calculate and set frequency */
        float2 = float1 * pow(RT12_2, note);
        spinbutton_freq_changed_by_user = FALSE;
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_freq), float2);
        spinbutton_freq_changed_by_user = TRUE;
    }
    else {
        notes_note = 0;
    }

    return;
}


