/*
 * pitchtune - an instrument tuner
 * Copyright (C) 2000 Haakon Andre Hjortland
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * \file draw.c
 * \brief Samplepieces drawing routine, included into tuner.c
 *
 * File to get included into tuner.c many times and with different #defines
 * so that we keep one general definition of the function, but with only the
 * size of the vars and the window size changed. (?)
 *
 * \todo Generalize display, and adapt to different sizes, not only functions
 * for two sizes: big and small.
 */


#ifndef DRAW_FUNC_BIG
#    error
#endif

#ifndef DRAW_FUNC_DEPTH
#    error
#endif


#if DRAW_FUNC_BIG != 0
#   if DRAW_FUNC_DEPTH == 8
        void draw_samplepieces_8_big()
#   elif DRAW_FUNC_DEPTH == 16
        void draw_samplepieces_16_big()
#   elif DRAW_FUNC_DEPTH == 32
        void draw_samplepieces_32_big()
#   endif
#else // DRAW_FUNC_BIG == 0
#   if DRAW_FUNC_DEPTH == 8
        void draw_samplepieces_8_small()
#   elif DRAW_FUNC_DEPTH == 16
        void draw_samplepieces_16_small()
#   elif DRAW_FUNC_DEPTH == 32
        void draw_samplepieces_32_small()
#   endif
#endif // DRAW_FUNC_BIG == 0

{
int i, j, k, l;
float float1;
GdkColormap *GdkColormap1; // a colormap
GdkColor GdkColor1; // a color

#if DRAW_FUNC_DEPTH == 8
guchar *image_buf;
guchar pixel_colors[SAMPLEPIECES_MAX];
guchar pixel_color_indicator_line;
guchar pixel_color_background;
guchar pixel_color_square;
#endif

#if DRAW_FUNC_DEPTH == 16
guint16 *image_buf;
guint16 pixel_colors[SAMPLEPIECES_MAX];
guint16 pixel_color_indicator_line;
guint16 pixel_color_background;
guint16 pixel_color_square;
#endif

#if DRAW_FUNC_DEPTH == 32
guint32 *image_buf;
guint32 pixel_colors[SAMPLEPIECES_MAX];
guint32 pixel_color_indicator_line;
guint32 pixel_color_background;
guint32 pixel_color_square;
#endif

image_buf=gdkimage1->mem;

displaystate_is_splash=FALSE;


GdkColormap1=gdk_window_get_colormap(drawingarea1->window);
gdk_colormap_ref(GdkColormap1);


if (samplepieces_count >= 2)
    for (i=0; i < samplepieces_count; i++)
	{
	    /* draw in green, and fade out when you increase samplepieces */
	    GdkColor1.red =   0;
	    GdkColor1.green = (65535*(samplepieces_count-i)) / samplepieces_count;
	    GdkColor1.blue =  0;
	    gdk_color_alloc(GdkColormap1, &GdkColor1);
	    pixel_colors[i]=GdkColor1.pixel;
        }
else
    {
    GdkColor1.red =   0;
    GdkColor1.green = 65535;
    GdkColor1.blue =  0;
    gdk_color_alloc(GdkColormap1, &GdkColor1);
    pixel_colors[0]=GdkColor1.pixel;
    }


// we want some RED for the lines we're going to display.
GdkColor1.red =   65535;
GdkColor1.green = 0;
GdkColor1.blue =  0;
gdk_color_alloc(GdkColormap1, &GdkColor1);
pixel_color_indicator_line=GdkColor1.pixel;


// this is a black color, for the background.
GdkColor1.red =   0;
GdkColor1.green = 0;
GdkColor1.blue =  0;
gdk_color_alloc(GdkColormap1, &GdkColor1);
pixel_color_background=GdkColor1.pixel;


// we want a white box around the small display.
GdkColor1.red =   65535;
GdkColor1.green = 65535;
GdkColor1.blue =  65535;
gdk_color_alloc(GdkColormap1, &GdkColor1);
pixel_color_square=GdkColor1.pixel;

// Clear (and maybe draw corners)
#if DRAW_FUNC_BIG != 0

for (i=0; i < DRAWAREA_HEIGHT*DRAWAREA_WIDTH; i++)
    image_buf[i]=pixel_color_background;
    display_cleared_around=FALSE;

#else // DRAW_FUNC_BIG == 0

if (display_cleared_around == FALSE)
    {
    for (i=0; i < DRAWAREA_HEIGHT*DRAWAREA_WIDTH; i++)
        image_buf[i]=pixel_color_background;
    }

for (i=DRAWAREA_HEIGHT_QUARTER*DRAWAREA_WIDTH;
    i < DRAWAREA_HEIGHT_QUARTER*3*DRAWAREA_WIDTH;
    i++)
    image_buf[i]=pixel_color_background;

// Draw square around the small display
for (i=DRAWAREA_WIDTH_QUARTER; i < DRAWAREA_WIDTH_QUARTER*3; i++)
    {
    image_buf[i+(DRAWAREA_HEIGHT_QUARTER-1)*DRAWAREA_WIDTH]=
        pixel_color_square;
    image_buf[i+(DRAWAREA_HEIGHT_QUARTER*3)*DRAWAREA_WIDTH]=
        pixel_color_square;
    }

for (i=DRAWAREA_HEIGHT_QUARTER; i < DRAWAREA_HEIGHT_QUARTER*3; i++)
    {
    image_buf[i*DRAWAREA_WIDTH+(DRAWAREA_WIDTH_QUARTER-1)]=
        pixel_color_square;
    image_buf[i*DRAWAREA_WIDTH+(DRAWAREA_WIDTH_QUARTER*3)]=
        pixel_color_square;
    }

#endif


// Draw samplepieces
for (i=samplepieces_last_valid;i>=0;i--)
    {
    k=(samplepieces_which_is_zero+i)%SAMPLEPIECES_MAX;
    k*=DRAWAREA_WIDTH;
    for (j=0; j < DRAWAREA_WIDTH; j++)
        {
#if DRAW_FUNC_BIG != 0
        l=j+DRAWAREA_WIDTH*samplepieces_buffer[k+j];
#else
        l=DRAWAREA_WIDTH_QUARTER+(j>>1)+DRAWAREA_WIDTH*(
            (samplepieces_buffer[k+j]>>1) + DRAWAREA_HEIGHT_QUARTER);
#endif
        image_buf[l]=pixel_colors[i];
        }
    }


// Draw indicator lines
float1 = 0.5;
j=-1;
for (i=0;i<DRAWAREA_WIDTH;i++)
    {
    label_draw_indicator_lines:
    if (i > j)
        {
        j = (int)floor( (wavelength_samples / downsample_ratio)
            * float1);
        float1++;
        goto label_draw_indicator_lines;
        }
    if (i == j)
#if DRAW_FUNC_BIG != 0
        for (k=0; k < DRAWAREA_HEIGHT; k++)
#else
        for (k=DRAWAREA_HEIGHT_QUARTER; k < DRAWAREA_HEIGHT_QUARTER*3; k++)
#endif
            if (k % 10 <= 1)
#if DRAW_FUNC_BIG != 0
                image_buf[i+k*DRAWAREA_WIDTH]=pixel_color_indicator_line;
#else
                image_buf[(i>>1)+DRAWAREA_WIDTH_QUARTER+k*DRAWAREA_WIDTH] =
                    pixel_color_indicator_line;
#endif
    }



#if DRAW_FUNC_BIG != 0
i=1;
#else
i=2;
if (display_cleared_around == FALSE)
    {
    display_cleared_around=TRUE;
    i=1;
    }
#endif

if (i == 1)
    gdk_draw_image(drawingarea1->window, GdkGC_black, gdkimage1,
        0, 0, 0, 0, DRAWAREA_WIDTH, DRAWAREA_HEIGHT);

if (i != 1)
    gdk_draw_image(drawingarea1->window, GdkGC_black, gdkimage1,
        DRAWAREA_WIDTH_QUARTER, DRAWAREA_HEIGHT_QUARTER,
        DRAWAREA_WIDTH_QUARTER, DRAWAREA_HEIGHT_QUARTER,
        DRAWAREA_WIDTH_QUARTER*2, DRAWAREA_HEIGHT_QUARTER*2);

gdk_flush();

gdk_colormap_unref(GdkColormap1);

return;
}


