/*
 * pitchtune - an instrument tuner
 * Copyright (C) 2000 Haakon Andre Hjortland
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * \file main.c
 * \brief int main() and some support functions
 */



/**** Includes */

/* define this if you want to use the old GtkText stuff, and not the new GtkTextView stuff... */
/*#define GTK_ENABLE_BROKEN */
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <gtk/gtk.h>
#include "main.h"

#include "main_img001.h"



/**** Initialize constants */
const char *(note_chars_lowercase_multi[][12]) =
{
    {"c", "c#", "d", "eb", "e", "f", "f#", "g", "g#", "a", "bb", "b"},
    {"c", "c#", "d", "d#", "e", "f", "f#", "g", "g#", "a", "a#", "h"}
};
const char *(note_chars_uppercase_multi[][12]) =
{
    {"C", "C#", "D", "Eb", "E", "F", "F#", "G", "G#", "A", "Bb", "B"},
    {"C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "H"}
};

/* Menu entry names */
const char *menu_entry[MENU_ENTRIES] =
{"mmenu_item_settings",
 "mmenu_item_exit",
 "mmenu_item_small_display",
 "mmenu_item_sel_guitar",
 "mmenu_item_about",
 "mmenu_item_help"};






/**
 * \brief Startup function
 *
 */
int main (int argc, char *argv[])
{
    GdkPixmap *priv_gdkpixmap1;
    GdkColormap *GdkColormap1;
    GdkColor GdkColor1;
    gchar dashlist[2];

    /* Initialize global variables */
    gstr_sounddev = g_string_new("");
    gstr_alsa_device = g_string_new("");
    scale_type = 0; /* Default defined elsewhere, this is just to be sure it's never */
                    /* used undefined */
    tuner_started = FALSE;
    samplepieces_count = -1;
    samplepieces_count_finetune = -1;
    samplepieces_count_coarsetune = -1;
    samplepieces_last_valid = -1;
    samplepieces_which_is_zero = 0;
    wavelength_samples = -1;
    downsample_ratio = -1;
    timer_tag = -1;
    timer_count = 0;
    sound_fd = -1;
    displaystate_is_splash = TRUE;
    display_big = TRUE;
    display_cleared_around = FALSE;
    sel_standard_note = 0;
    sel_standard_octave = 0;
    sel_standard_last_octave = 0;
    sel_standard_last_octave_valid = FALSE;
    sel_standard_last_scale_type = 0;
    samplerate = -1;
    window_settings = NULL;
    fileselection1 = NULL;
    about_dialog = NULL;
    help_dialog = NULL;
    debug_window = NULL;
    debug_text_box = NULL;
    soundbuffer_len = 0;
    soundbuffer_readpos = 0;
    soundbuffer_writepos = 0;
    frequency_set = FALSE;
    backto_lastdraw_samplecount = 0.0;
    spinbutton_freq_changed_by_user = TRUE;
    button_muting1 = NULL;


    /* Read options */
    {
	int i;
	for (i = 0; i < argc; i++)
	    {
		if ((strcmp(argv[i], "--help") == 0) ||
		    (strcmp(argv[i], "-h") == 0) ||
		    (strcmp(argv[i], "-?") == 0))
		    {
			printf("Usage: pitchtune [OPTION]...\n");
			printf("A GPL'ed GTK oscilloscope-style musical "
			       "instrument tuning program\n");
			printf("    --help    display this help screen and exit\n");
			printf("\n");
			printf("Report bugs to <hahjortland@tande.com>\n");
			return 0;
		    }
	    }
    }

    /* Gtk initialization */
    gtk_set_locale();
    gtk_init(&argc, &argv);

    /* From Glade. Currently not needed. */
    add_pixmap_directory("./pixmaps");
    add_pixmap_directory("../pixmaps");


    /* Create and prepare debug window */
    debug_window=create_debug_window();
    debug_text_box=lookup_widget(debug_window, "debug_text_box");


    /* Load configuration file */
    config_load();


    /* Create and prepare main window */
    window_main = create_window_main ();
    mmenu_item_sel_guitar = lookup_widget (window_main, "mmenu_item_sel_guitar");
    mmenu_item_small_display = lookup_widget (window_main,
					      "mmenu_item_small_display");
    drawingarea1 = lookup_widget (window_main, "drawingarea1");
    spinbutton_freq = lookup_widget (window_main, "spinbutton_freq");
    spinbutton_a1freq = lookup_widget (window_main, "spinbutton_a1freq");
    optionmenu_noteselect=lookup_widget (window_main, "optionmenu_noteselect");
    optionmenu_noteselect_menu=
	gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu_noteselect));
    checkbutton_ministep_freq = lookup_widget (window_main, "checkbutton_ministep_freq");
    checkbutton_ministep_a1freq = lookup_widget (window_main, "checkbutton_ministep_a1freq");
    checkbutton_coarse_tuning = lookup_widget (window_main, "checkbutton_coarse_tuning");
    hscale_finetune = lookup_widget (window_main, "hscale_finetune");
    adjustment_finetune = gtk_range_get_adjustment (GTK_RANGE(hscale_finetune));
    hscale_coarsetune = lookup_widget (window_main, "hscale_coarsetune");
    adjustment_coarsetune = gtk_range_get_adjustment (GTK_RANGE(hscale_coarsetune));
    button_muting1 = lookup_widget(window_main, "muting1");

    label_freqwarn = lookup_widget (window_main, "label_freqwarn");
    gtk_label_set_text(GTK_LABEL(label_freqwarn), " ");

    gtk_signal_connect (GTK_OBJECT(optionmenu_noteselect_menu), "deactivate",
			GTK_SIGNAL_FUNC(on_optionmenu_noteselect_menu_deactivate), NULL);
    gtk_signal_connect (GTK_OBJECT(adjustment_finetune), "value_changed",
			GTK_SIGNAL_FUNC(on_adjustment_finetune_value_changed), NULL);
    gtk_signal_connect (GTK_OBJECT(adjustment_coarsetune), "value_changed",
			GTK_SIGNAL_FUNC(on_adjustment_coarsetune_value_changed), NULL);

    /* Realize main window and drawingarea1 */
    gtk_widget_realize(window_main);
    gtk_widget_realize(drawingarea1);


    /* Initialize note selectors */
    sel_guitar_init();


    /* Set configurations */

    /* Set keyboard accelerators */

/*
temp. comment all keyboard accel stuff, until someone really needs it.
Why isn't hard coded anyway ?
    {
	GtkAccelGroup *accel_group;
	int i;
	GtkWidget *menu_item;

	accel_group = gtk_accel_group_new ();

	for (i = 0; i < MENU_ENTRIES; i++)
	    {
		menu_item = lookup_widget (window_main, menu_entry[i]);
		if ((config.menu_accel_key[i] >= 0) && (config.menu_accel_mods[i] >= 0))
		    gtk_widget_add_accelerator (menu_item, "activate", accel_group,
						config.menu_accel_key[i], config.menu_accel_mods[i],
						GTK_ACCEL_VISIBLE);
	    }
    }
*/

/* Set a1-freq, freq, note and fine/coarse decays, muting */
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_a1freq), config.a1frequency);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_freq), config.frequency);
    set_note(config.note_set, config.note);
    gtk_adjustment_set_value(adjustment_finetune, (float)config.decay_fine);
    gtk_adjustment_set_value(adjustment_coarsetune, (float)config.decay_coarse);
    gtk_toggle_button_set_active(button_muting1, config.play_sine);


    /* Set main window position */
    if (main_config.window.xpos < 0) main_config.window.xpos=0;
    if (main_config.window.ypos < 0) main_config.window.ypos=0;
    if (main_config.window.pos_set == TRUE)
	gtk_widget_set_uposition(window_main,
				 main_config.window.xpos, main_config.window.ypos);

    /* Set display size (big/small) */
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mmenu_item_small_display),
				   !config.display_big);


    /* Init some color-stuff */
    GdkColormap1=gdk_window_get_colormap(drawingarea1->window);
    gdk_colormap_ref(GdkColormap1);
    gdkvisual1=gdk_window_get_visual(drawingarea1->window);
    gdk_visual_ref(gdkvisual1);

    /* GdkGC_red */
    GdkGC_red = gdk_gc_new (drawingarea1->window);
    GdkColor1.red =   65535;
    GdkColor1.green = 0;
    GdkColor1.blue =  0;
    gdk_color_alloc (GdkColormap1, &GdkColor1);
    gdk_gc_set_foreground (GdkGC_red, &GdkColor1);
    gdk_gc_set_line_attributes(GdkGC_red, 1, GDK_LINE_ON_OFF_DASH,
			       GDK_CAP_ROUND, GDK_JOIN_ROUND);
    dashlist[0]=1;
    dashlist[1]=10;
    gdk_gc_set_dashes(GdkGC_red, 0, dashlist, 2);

    /* GdkGC_black */
    GdkGC_black = gdk_gc_new (drawingarea1->window);
    GdkColor1.red =   0;
    GdkColor1.green = 0;
    GdkColor1.blue =  0;
    gdk_color_alloc (GdkColormap1, &GdkColor1);
    gdk_gc_set_foreground (GdkGC_black, &GdkColor1);

    gdk_window_set_background(drawingarea1->window, &GdkColor1);

    /* Unref GdkColormap1 and gdkvisual1 */
    gdk_colormap_unref(GdkColormap1);
    gdk_visual_unref(gdkvisual1);


    /* Create gdkpixmap1 and make splash-screen */
    gdkpixmap1=gdk_pixmap_new(drawingarea1->window,
			      DRAWAREA_WIDTH, DRAWAREA_HEIGHT, -1);
    priv_gdkpixmap1=gdk_pixmap_create_from_xpm_d(drawingarea1->window,
						 NULL, NULL, main_img001);
    gdk_draw_pixmap(gdkpixmap1, GdkGC_black, priv_gdkpixmap1,
		    0, 0, 0, 0, DRAWAREA_WIDTH, DRAWAREA_HEIGHT);


    /* Create GdkImage for drawing waves on */
    gdkimage1=gdk_image_new(GDK_IMAGE_FASTEST, gdkvisual1,
			    DRAWAREA_WIDTH, DRAWAREA_HEIGHT);


    /* Maybe show note selectors */
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mmenu_item_sel_guitar),
				   config.sel_guitar_visible);


    /* Show main window */
    gtk_widget_show(GTK_WIDGET(window_main));


    /* Start GTK main loop */
    gtk_main();
    return(0);
}



/*
 * double_modulus
 *
 * Modulus, but for floating-point data
 *
 */

double double_modulus(double a, double b)
{
    return( a - (b * floor(a / b)) );
}



/**
 * \brief Quit the program, saving configuration
 */
void exit_program()
{
    /* Save configuration */
    config_save();

    /* Exit */
    gtk_exit(0);
    return;
}



/*
 * config_load
 *
 */

void config_load()
{
    FILE *file1;
    char *str1;
    char *str2;
    char *str3;
    const char *str4;
    int int1, int2, int3, int4;
    int i;
    double double1;
    gboolean int4_ok;
    gboolean double1_ok;
    GString *gstr1;

    /* DEFAULT VALUES FOR CONFIG SETTINGS */
    config.sel_guitar_visible=FALSE;
    config.a1frequency = 440.0; /* Default a1-frequency */
    config.frequency = 123.0; /* Default frequency */
    config.note_set = TRUE;
    config.note = 0;
    config.decay_coarse = 3;
    config.decay_fine = 15;
    config.display_big = TRUE;
    config.play_sine = 0;
    g_string_sprintf(gstr_sounddev, "/dev/dsp");
    g_string_sprintf(gstr_alsa_device, "default");
    scale_type=0;

    for (i = 0; i < MENU_ENTRIES; i++)
	{
	    config.menu_accel_key[i] = -1;
	    config.menu_accel_mods[i] = -1;
	}

    if ((str1 = malloc(1001)) == NULL)
	{
	    PRINTF_DEBUG("Error allocating memory\n");
	    return;
	}
    if ((str2 = malloc(1001)) == NULL)
	{
	    PRINTF_DEBUG("Error allocating memory\n");
	    return;
	}
    if ((str3 = malloc(1001)) == NULL)
	{
	    PRINTF_DEBUG("Error allocating memory\n");
	    return;
	}

    gstr1 = g_string_new("");

    /* Clear configuration variables */
    sel_guitar_config.window.xpos = 1;
    sel_guitar_config.window.ypos = 1;
    sel_guitar_config.window.pos_set = FALSE;
    main_config.window.xpos = 1;
    main_config.window.ypos = 1;
    main_config.window.pos_set = FALSE;

    /* Read configuration file */
    str4=getenv("HOME");
    if (str4 == NULL)
	{
	    PRINTF_DEBUG(
		"Could not read config-file (could not find HOME directory)\n");
	    goto label3;
	}

    g_string_sprintf(gstr1, "%s%s", str4, "/.pitchtune");
    file1 = fopen(gstr1->str, "r");

    if (file1 == NULL) {
        PRINTF_DEBUG("Could not open config-file\n");
        goto label3;
    }

    /* Read next line */
  label1:

    /* read a line */
    str1[0] = 0;
    fscanf(file1, "%1000s", str1);

    /* skip empty lines */
    if (strlen(str1) == 0)
        goto label2;

    int2 = strlen(str1);
    int3 = -1;

    for (int1 = 0 ; int1 < int2 ; int1++) {
        /* set '=' position */
        if ((str1[int1] == '=') && (int3 == -1))
            int3 = int1;
    }

    /* skip lines with no '=' sign */
    if (int3 == -1) goto label1;

    /* copy var name in str2 */
    strncpy(str2, str1, int3);
    str2[int3] = 0;

    /* put value in str3 */
    strcpy(str3, &(str1[int3+1]));

    int4_ok = FALSE;
    double1_ok = FALSE;

    /* try to read an int */
    int1 = sscanf(str3, "%d", &int4);
    if (int1 == 1)
      int4_ok = TRUE;

    /* try to read a double */
    int1 = sscanf(str3, "%lf", &double1);
    if (int1 == 1)
      double1_ok = TRUE;


    /* Interprete the line */

    /* sel_guitar */
    if ((strcmp(str2, "pitchtune.sel_guitar.window.xpos") == 0) &&
	(int4_ok == TRUE))
        {
	    sel_guitar_config.window.xpos=int4;
	    sel_guitar_config.window.pos_set=TRUE;
        }

    if ((strcmp(str2, "pitchtune.sel_guitar.window.ypos") == 0) &&
	(int4_ok == TRUE))
        {
	    sel_guitar_config.window.ypos=int4;
	    sel_guitar_config.window.pos_set=TRUE;
        }

    if ((strcmp(str2, "pitchtune.sel_guitar.window.visible") == 0) &&
	(int4_ok == TRUE))
        {
	    if (int4 == 0)
		config.sel_guitar_visible=FALSE;
	    else
		config.sel_guitar_visible=TRUE;
        }

    /* Main window */
    if ((strcmp(str2, "pitchtune.main.window.xpos") == 0) &&
	(int4_ok == TRUE))
        {
	    main_config.window.xpos=int4;
	    main_config.window.pos_set=TRUE;
        }

    if ((strcmp(str2, "pitchtune.main.window.ypos") == 0) &&
	(int4_ok == TRUE))
        {
	    main_config.window.ypos=int4;
	    main_config.window.pos_set=TRUE;
        }

    /* Keyboard accelerators */
/*
don't lose your time with keyboard accels 
    {
	GString *gstr;
	gstr = g_string_new("");

	for (i = 0; i < MENU_ENTRIES; i++)
	    {
		g_string_sprintf(gstr, "pitchtune.main.menu_accel.%s.key", menu_entry[i]);
		if ((strcmp(str2, gstr->str) == 0) &&
		    (int4_ok == TRUE))
		    config.menu_accel_key[i] = int4;
		g_string_sprintf(gstr, "pitchtune.main.menu_accel.%s.mods", menu_entry[i]);
		if ((strcmp(str2, gstr->str) == 0) &&
		    (int4_ok == TRUE))
		    config.menu_accel_mods[i] = int4;
	    }

	g_string_free(gstr, TRUE);
    }
*/

/* Big display */
    if ((strcmp(str2, "pitchtune.main.display_big") == 0) &&
	(int4_ok == TRUE))
        {
	    if (int4 == 0)
		config.display_big=FALSE;
	    else
		config.display_big=TRUE;
        }

    /* a1-freq, freq and note */
    if ((strcmp(str2, "pitchtune.main.a1frequency") == 0) &&
	(double1_ok == TRUE))
        {
	    config.a1frequency=double1;
        }

    if ((strcmp(str2, "pitchtune.main.frequency") == 0) &&
	(double1_ok == TRUE))
        {
	    config.frequency=double1;
        }

    if ((strcmp(str2, "pitchtune.main.note_set") == 0) &&
	(int4_ok == TRUE))
        {
	    if (int4 == 0)
		config.note_set=FALSE;
	    else
		config.note_set=TRUE;
        }

    if ((strcmp(str2, "pitchtune.main.note") == 0) &&
	(int4_ok == TRUE))
        {
	    config.note=int4;
        }

    /* Fine/coarse decays */
    if ((strcmp(str2, "pitchtune.main.decay.coarse") == 0) &&
	(int4_ok == TRUE))
        {
	    config.decay_coarse=int4;
        }

    if ((strcmp(str2, "pitchtune.main.decay.fine") == 0) &&
	(int4_ok == TRUE))
        {
	    config.decay_fine=int4;
        }

    /* Sound device */
    if (strcmp(str2, "pitchtune.main.sounddevice") == 0)
	{
	    g_string_sprintf(gstr_sounddev, "%s", str3);
	}

    if (strcmp(str2, "pitchtune.main.alsadevice") == 0)
	{
	    g_string_sprintf(gstr_alsa_device, "%s", str3);
	}

    /* Scale type */
    if ((strcmp(str2, "pitchtune.main.scale_type") == 0) &&
	(int4_ok == TRUE))
        {
	    if ((int4 >= 0) && (int4 < SCALE_COUNT))
		scale_type=int4;
        }

    /* Muting (sine wave) */
    if ((strcmp(str2, "pitchtune.main.play_sine") == 0) &&
        (int4_ok == TRUE))
        {
            config.play_sine = (int4 ? 1 : 0);
        }


    goto label1;

    /* Finished reading file */
  label2:
    fclose(file1);

  label3:

    free(str1);
    free(str2);
    free(str3);
    g_string_free(gstr1, TRUE);

    return;
}



/*
 * config_save
 *
 */

void config_save()
{
    FILE *file1;
    int int1, int2;
    double double1;
    const char *str1;
    GString *gstr1;
    gstr1=g_string_new("");

    str1=getenv("HOME");
    if (str1 == NULL)
	{
	    PRINTF_DEBUG(
		"Could not write config-file (could not find HOME directory)\n");
	    goto return_from_func;
	}

    g_string_sprintf(gstr1, "%s%s", str1, "/.pitchtune");
    file1=fopen(gstr1->str, "w");

    if (file1 == NULL)
	{
	    PRINTF_DEBUG("Could not write config file (could not open file)\n");
	    goto return_from_func;
	}

    /* Main window */
    gdk_window_get_root_origin(window_main->window, &int1, &int2);
    fprintf(file1, "pitchtune.main.window.xpos=%d\n", int1);
    fprintf(file1, "pitchtune.main.window.ypos=%d\n", int2);
    int1=(display_big)?1:0;
    fprintf(file1, "pitchtune.main.display_big=%d\n", int1);

    /* Keyboard accelerators */

/*  temporarily disable keyboard accels, woah with new Gtk!
    {
	int key;
	int mods;
	int i;
	GSList *slist;
	GtkWidget *menu_item;
	GtkWidget *accel_label;
	GtkWidget *accel_widget;
	GtkAccelEntry *entry;

	for (i = 0; i < MENU_ENTRIES; i++)
	    {
		menu_item = lookup_widget (window_main, menu_entry[i]);
		accel_label = GTK_BIN(menu_item)->child;
		accel_widget = GTK_ACCEL_LABEL(accel_label)->accel_widget;
		slist = gtk_accel_group_entries_from_object (GTK_OBJECT(accel_widget));
		key = -1;
		mods = -1;
		for (; slist; slist = slist->next)
		    {
			entry = slist->data;
			if (entry->accel_flags & GTK_ACCEL_VISIBLE)
			    {
				key = entry->accelerator_key;
				mods = entry->accelerator_mods;
			    }
		    }
		if (key >= 0)
		    {
			fprintf(file1, "pitchtune.main.menu_accel.%s.key=%d\n", menu_entry[i],
				key);
			fprintf(file1, "pitchtune.main.menu_accel.%s.mods=%d\n", menu_entry[i],
				mods);
		    }
	    }
    }
*/

/* Sel_guitar */
    if (sel_guitar_window_visible)
	{
	    gdk_window_get_root_origin(sel_guitar_window->window, &int1, &int2);
	    sel_guitar_config.window.xpos=int1;
	    sel_guitar_config.window.ypos=int2;
	    sel_guitar_config.window.pos_set=TRUE;
	}

    if (sel_guitar_config.window.pos_set == TRUE)
	{
	    fprintf(file1, "pitchtune.sel_guitar.window.xpos=%d\n",
		    sel_guitar_config.window.xpos);
	    fprintf(file1, "pitchtune.sel_guitar.window.ypos=%d\n",
		    sel_guitar_config.window.ypos);
	}
    else
	{
	    fprintf(file1, "pitchtune.sel_guitar.window.xpos=?\n");
	    fprintf(file1, "pitchtune.sel_guitar.window.ypos=?\n");
	}

    if (sel_guitar_window_visible)
	fprintf(file1, "pitchtune.sel_guitar.window.visible=1\n");
    else
	fprintf(file1, "pitchtune.sel_guitar.window.visible=0\n");

    /* a1-freq, freq and note */
    double1=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_a1freq));
    fprintf(file1, "pitchtune.main.a1frequency=%f\n", double1);

    double1=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_freq));
    fprintf(file1, "pitchtune.main.frequency=%f\n", double1);

    if (notes_note_set == TRUE)
	fprintf(file1, "pitchtune.main.note_set=1\n");
    else
	fprintf(file1, "pitchtune.main.note_set=0\n");

    fprintf(file1, "pitchtune.main.note=%d\n", notes_note);

    /* Fine/coarse decays */
    int1=(int)(adjustment_finetune->value);
    fprintf(file1, "pitchtune.main.decay.fine=%d\n", int1);

    int1=(int)(adjustment_coarsetune->value);
    fprintf(file1, "pitchtune.main.decay.coarse=%d\n", int1);

    /* Sound device */
    fprintf(file1, "pitchtune.main.sounddevice=%s\n", gstr_sounddev->str);
    fprintf(file1, "pitchtune.main.alsadevice=%s\n", gstr_alsa_device->str);

    /* Scale type */
    fprintf(file1, "pitchtune.main.scale_type=%d\n", scale_type);

    /* Sine wave muting state */
    fprintf(file1, "pitchtune.main.play_sine=%d\n", (int) config.play_sine);

    /* Close file */
    fclose(file1);


  return_from_func:
    g_string_free(gstr1, TRUE);

    return;
}



/*
 * print_to_debug_window
 *
 * Prints to a debugging-window (displayed when necessary) and stderr
 *
 */
void print_to_debug_window(char *str)
{
    int int1;
    int int2;
    char char1;
    GtkTextView *debugtext;

    /* Also print debug information to stderr */
    g_printerr("pitchtune: %s", str);

    /* Show debug window and add the new debug text */
/* tests to make that thing work with Gtk2 */
    gtk_widget_show(debug_window);
    debugtext = GTK_TEXT_VIEW(debug_text_box);
    gtk_signal_emit_by_name(GTK_OBJECT(debug_text_box), "GtkTextView::insert_at_cursor", str);
    debugtext->editable = FALSE;
    /*    gtk_text_insert(GTK_TEXT_it uVIEW(debug_text_box), NULL, NULL, NULL, str, -1); */
    /*    gtk_text_freeze(GTK_TEXT_VIEW(debug_text_box)); */


    /**
     * \todo Complete this section with new GtkTextView API
     */

/*
// Find out if there's too much text in the debug window
  label0:
    for (int1=0; debugtext->GTK_TEXT_INDEX(GTK_TEXT_VIEW(debug_text_box), int1) != 0; int1++);

    if (int1 > 4000)
	{
	    // Too much text in display, remove some text
    
	    for (int2=0; int2 < int1; int2++)
		{
		    char1=GTK_TEXT_INDEX(GTK_TEXT_VIEW(debug_text_box), int2);
		    if (char1 == '\n') goto label1;
		}
    
	  label1:
	    if (int2 >= int1) int2=int1-1;
	    // Delete one line
	    gtk_text_set_point(GTK_TEXT(debug_text_box), 0);
	    gtk_text_forward_delete(GTK_TEXT(debug_text_box), int2+1);
    
	    goto label0;
	}
    else
	{
	    // Set pointer to end of the text
	    gtk_text_set_point(GTK_TEXT(debug_text_box), int1);
	}

    gtk_text_thaw(GTK_TEXT(debug_text_box));
*/

    return;
}

