/*
 * pitchtune - an instrument tuner
 * Copyright (C) 2000 Haakon Andre Hjortland
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


/**
 * \file sel_guitar.c
 * \brief Guitar note selector window functions
 * \todo Document the functions inside this file
 */



/**** Incldues */

#include <gdk/gdkkeysyms.h>
#include "main.h"
#include "sel_guitar_img001.h"



/**
 * \brief Note numbers of the 6 guitar strings (E,A,D,G,H,E)
 * Only used inside sel_guitar.c, so no need to put it somewhere else,
 * it causes troubles compiling anyway.
 */
int sel_guitar_string_notes[] = {-29, -24, -19, -14, -10, -5};


/**
 * \brief Initialize the guitar note selector
 */
void sel_guitar_init()
{
    /* Variable initialization */
    sel_guitar_window_visible=FALSE;
    sel_guitar_last_position=-1;

    /* Create window */
    /**
     * \todo Check this out, replaced GTK_WINDOW_DIALOG with GTK_WINDOW_TOPLEVEL for Gtk2 compatibility
     */
    sel_guitar_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(sel_guitar_window), "Guitar");
    gtk_window_set_policy(GTK_WINDOW(sel_guitar_window), FALSE, FALSE, TRUE);
    gtk_signal_connect(GTK_OBJECT(sel_guitar_window), "delete_event",
		       GTK_SIGNAL_FUNC(sel_guitar_on_window1_delete_event), NULL);
    gtk_widget_realize(sel_guitar_window);

    /* Create a gdk-pixmap with an image in it */
    sel_guitar_gdk_pixmap_picture=gdk_pixmap_create_from_xpm_d(
	sel_guitar_window->window, NULL, NULL, sel_guitar_img001);
    sel_guitar_gdk_pixmap1=gdk_pixmap_create_from_xpm_d(
	sel_guitar_window->window, NULL, NULL, sel_guitar_img001);

    /* Create a gtk-pixmap containing the previously created gdk-pixmap */
    sel_guitar_gtk_pixmap1=gtk_pixmap_new(sel_guitar_gdk_pixmap1, NULL);

    /* Create a eventbox to put the gtk-pixmap in */
    sel_guitar_eventbox1=gtk_event_box_new();
    gtk_widget_set_events(sel_guitar_eventbox1,
			  GDK_BUTTON_PRESS_MASK | GDK_KEY_PRESS_MASK);

    /* Put the pixmap in the event box */
    gtk_container_add(GTK_CONTAINER(sel_guitar_eventbox1), sel_guitar_gtk_pixmap1);

    /* Put the eventbox in the window */
    gtk_container_add(GTK_CONTAINER(sel_guitar_window), sel_guitar_eventbox1);

    /* Show widgets */
    gtk_widget_show(sel_guitar_gtk_pixmap1);
    gtk_widget_show(sel_guitar_eventbox1);

    /* Let eventbox be focusable, and grab focus */
    GTK_WIDGET_SET_FLAGS(sel_guitar_eventbox1, GTK_CAN_FOCUS);
    gtk_widget_grab_focus(GTK_WIDGET(sel_guitar_eventbox1));

    /* Connect signals */
    gtk_signal_connect(GTK_OBJECT(sel_guitar_eventbox1), "button-press-event",
		       GTK_SIGNAL_FUNC(sel_guitar_on_eventbox1_button_press_event), NULL);

    gtk_signal_connect(GTK_OBJECT(sel_guitar_eventbox1), "key-press-event",
		       GTK_SIGNAL_FUNC(sel_guitar_on_eventbox1_key_press_event), NULL);

    /* Realize window */
    gtk_widget_realize(sel_guitar_window);

    /* Draw graphics */
    sel_guitar_set_note(FALSE, 0);


    return;
}



/**
 * \brief Show guitar selector window
 */
void sel_guitar_show_window()
{
    sel_guitar_window_visible=TRUE;

    /* Restore window positions */
    if (sel_guitar_config.window.xpos < 0) sel_guitar_config.window.xpos=0;
    if (sel_guitar_config.window.ypos < 0) sel_guitar_config.window.ypos=0;
    if (sel_guitar_config.window.pos_set == TRUE)
	gtk_widget_set_uposition(sel_guitar_window,
				 sel_guitar_config.window.xpos, sel_guitar_config.window.ypos);

    /* Show window */
    gtk_widget_show(sel_guitar_window);

    /* Update graphics */
    sel_guitar_set_note(notes_note_set, notes_note);

    return;
}



/**
 * \brief Hide guitar selector window
 */
void sel_guitar_hide_window()
{
    int int1, int2;

    if (sel_guitar_window_visible == FALSE) return;
    sel_guitar_window_visible=FALSE;

    /* Save window position */
    gdk_window_get_root_origin(sel_guitar_window->window, &int1, &int2);
    sel_guitar_config.window.xpos=int1;
    sel_guitar_config.window.ypos=int2;
    sel_guitar_config.window.pos_set=TRUE;

    /* Hide window */
    gtk_widget_hide(sel_guitar_window);

    /* Inform the main program about the change */
    sel_guitar_set_active(FALSE);

    return;
}



/**
 * \brief Hit the hide button, call hide window
 *
 * Only a mapping between Gtk stuff and the hide function itself.
 */
gboolean sel_guitar_on_window1_delete_event(GtkWidget *widget,
					    GdkEvent *event, gpointer user_data)
{
    sel_guitar_hide_window();

    return(TRUE);
}



/**
 * \brief Select note using the guitar display, and choose note using
 *        the string number.
 * \param pos Guitar string number (0 to 5), 0 being the lowest pitch.
 *
 * This will set the current note to one of the six notes takes
 * from the sel_guitar_string_notes[] array.
 */
void sel_guitar_set_note_by_pos(int pos)
{
    /* Make sure it's between 0 and 5 */
    if ((pos >= 0) && (pos <= 5))
	set_note(TRUE, sel_guitar_string_notes[pos]);
    else
	set_note(FALSE, 0);

    return;
}



/*
 * sel_guitar_on_eventbox1_button_press_event
 *
 */
gboolean sel_guitar_on_eventbox1_button_press_event(GtkWidget *widget,
						    GdkEventButton *event, gpointer user_data)
{
    int pos=-1;

    if ((event->x > 190) && (event->x < 370))
	{
	    if ((event->y >  50) && (event->y <=  65)) pos=5;
	    if ((event->y >  65) && (event->y <=  80)) pos=4;
	    if ((event->y >  80) && (event->y <=  97)) pos=3;
	    if ((event->y >  97) && (event->y <= 114)) pos=2;
	    if ((event->y > 114) && (event->y <= 130)) pos=1;
	    if ((event->y > 130) && (event->y <= 146)) pos=0;
	}

    if (pos != -1) sel_guitar_set_note_by_pos(pos);

    return(TRUE);
}



/*
 * sel_guitar_on_eventbox1_key_presss_event
 *
 */
gboolean sel_guitar_on_eventbox1_key_press_event(GtkWidget *widget,
						 GdkEventKey *event, gpointer user_data)
{
    int pos;
    int int1;

    /* Find which string-position the current note corresponds to */
    pos=-1;
    if (notes_note_set == TRUE)
	for (int1=0; int1 <= 5; int1++)
	    if (notes_note == sel_guitar_string_notes[int1])
		pos=int1;

    /* Interprete keypress */
    if (event->keyval == GDK_Up)
	{
	    gtk_signal_emit_stop_by_name(GTK_OBJECT(sel_guitar_eventbox1),
					 "key-press-event");
	    if (pos == -1)
		pos=0;
	    else
		pos++;
	}
    else if (event->keyval == GDK_Down)
	{
	    gtk_signal_emit_stop_by_name(GTK_OBJECT(sel_guitar_eventbox1),
					 "key-press-event");
	    if (pos == -1)
		pos=5;
	    else
		pos--;
	}
    else
	return(FALSE); /* Keypress is not for us, we didn't handle it */

    if (pos < 0) pos=0;
    if (pos > 5) pos=5;

    sel_guitar_set_note_by_pos(pos);

    return(TRUE);
}



/*
 * sel_guitar_set_note
 *
 */
void sel_guitar_set_note(gboolean note_set, int note)
{
    GdkGC *GdkGC1;
    GdkColormap *GdkColormap1;
    GdkColor GdkColor1;
    int int1;
    int pos;

    /* Only update graphics if window is visible */
    if (sel_guitar_window_visible == FALSE) return;

    /* Make color */
    GdkColormap1 = gdk_window_get_colormap (sel_guitar_window->window);
    gdk_colormap_ref(GdkColormap1);

    GdkGC1 = gdk_gc_new (sel_guitar_window->window);
    GdkColor1.red =   65535;
    GdkColor1.green = 0;
    GdkColor1.blue =  0;
    gdk_color_alloc (GdkColormap1, &GdkColor1);

    gdk_gc_set_foreground (GdkGC1, &GdkColor1);

    /* Find which guitar-position the note corresponds to */
    pos=-1;
    if (note_set == TRUE)
	for (int1=0; int1 <= 5; int1++)
	    if (note == sel_guitar_string_notes[int1])
		pos=int1;

    /* If positon has changed, update display */
    if (pos != sel_guitar_last_position)
	{
	    gdk_draw_pixmap(sel_guitar_gdk_pixmap1, GdkGC1,
			    sel_guitar_gdk_pixmap_picture, 200, 56, 200, 56, 160, 90);

	    if (pos==5)
		gdk_draw_rectangle(sel_guitar_gdk_pixmap1, GdkGC1, TRUE, 200,  56, 160,  3);
	    else if (pos==4)
		gdk_draw_rectangle(sel_guitar_gdk_pixmap1, GdkGC1, TRUE, 200,  71, 160,  3);
	    else if (pos==3)
		gdk_draw_rectangle(sel_guitar_gdk_pixmap1, GdkGC1, TRUE, 200,  87, 160,  3);
	    else if (pos==2)
		gdk_draw_rectangle(sel_guitar_gdk_pixmap1, GdkGC1, TRUE, 200,  104, 160,  3);
	    else if (pos==1)
		gdk_draw_rectangle(sel_guitar_gdk_pixmap1, GdkGC1, TRUE, 200,  121, 160,  3);
	    else if (pos==0)
		gdk_draw_rectangle(sel_guitar_gdk_pixmap1, GdkGC1, TRUE, 200,  137, 160,  3);
    
	    gtk_widget_queue_draw_area(GTK_WIDGET(sel_guitar_gtk_pixmap1),
				       0, 0, 370, 150);
    
	    sel_guitar_last_position=pos;
	}

    gdk_gc_unref(GdkGC1);
    gdk_colormap_unref(GdkColormap1);

    return;
}


