/*
 * pitchtune - an instrument tuner
 * Copyright (C) 2000 Haakon Andre Hjortland
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * \file ui.c
 * \brief User interface routines
 */



//*** Includes

#include "main.h"



/**
 * \brief Check if display must be completely redrawn (including downsampling)
 */
void check_redo_display()
{
    gboolean bool1;
    
    // Coarse tuning?
    bool1 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_coarse_tuning));

    if ((tuner_started == FALSE) && (displaystate_is_splash == FALSE)) {
        if (bool1 == TRUE)
            process_data(PROCESS_DATA_REDO_SMALLSTEP, TRUE);
        else
            process_data(PROCESS_DATA_REDO_BIGSTEP, FALSE);
    }

    return;
}



/*
 * GTK callback function
 *
 */
void on_adjustment_finetune_value_changed (GtkAdjustment *adjustment,
                                           gpointer user_data)
{
    set_samplepieces_count_finetune(adjustment->value);
    return;
}



/*
 * GTK callback function
 *
 */

void on_adjustment_coarsetune_value_changed (GtkAdjustment *adjustment,
                                             gpointer user_data)
{
    set_samplepieces_count_coarsetune(adjustment->value);
    return;
}



/*
 * sel_standar_set_note
 *
 * Standard note selector: set_note
 *
 */

void sel_standard_set_note(gboolean note_set, int note)
{
    int int1, int2;

    if (note_set == TRUE) {
        int1 = (note + 9 + 12000) % 12;
        int2 = (note - int1 + 9 ) / 12;
        sel_standard_note = int1;
        sel_standard_octave = int2;
        sel_standard_update(FALSE);
    }
    else {
        sel_standard_note = -1;
        sel_standard_update(FALSE);
    }

    return;
}



/**
 * \brief Create and display the note name with the octave super/subscript'd
 *
 * Creates a GTK-widget which shows a note
 *
 * \param note Note number
 */
GtkWidget *create_note_label(int note)
{
    int int1;
    int int2;
    int int3;
    GString *mystr;
    GtkWidget *label1;

    int1 = note + 9;
    int2 = (int1 + 12000) % 12; // Note
    int3 = (int1 - int2) / 12; // Octave

    mystr = g_string_new("");


    /* Add note name */
    g_string_sprintf(mystr, "%s", (int3 >= -1 ?
                                   note_chars_lowercase_multi[scale_type][int2] :
                                   note_chars_uppercase_multi[scale_type][int2]));
    
    if (int3 >= 0)
        g_string_append_printf(mystr, "<sup>%d</sup>", int3 + 1);

    else if (int3 <= -3)
        g_string_append_printf(mystr, "<sub>%d</sub>", -int3 - 2);

    label1 = gtk_label_new(NULL);
    gtk_label_set_markup(label1, mystr->str);

    gtk_widget_show(label1);

    g_string_free(mystr, TRUE);

    return(label1);
}



/*
 * GTK callback function
 *
 */
void on_optionmenu_noteselect_menu_deactivate(GtkMenuShell *menu_shell,
                                              gpointer data)
{
    GtkWidget *widget1;
    int int1;

    widget1 = gtk_menu_get_active(GTK_MENU(optionmenu_noteselect_menu));
    int1 = g_list_index(GTK_MENU_SHELL(optionmenu_noteselect_menu)->children,
                        widget1);

    sel_standard_note = int1 - 1;

    sel_standard_inform_set_note();

    return;
}



/*
 * sel_standard_update
 *
 * Update optionmenu_noteselect
 *
 * inform_set_note: send set_note the update information?
 *
 */
void sel_standard_update(gboolean inform_set_note)
{
    GtkWidget *wid1;
    GtkWidget *menu1;
    GtkWidget *menuitem1;
    int int2, int3;


    // Only make new menu if octave has changed
    if (!((sel_standard_last_octave_valid == TRUE) &&
          (sel_standard_octave == sel_standard_last_octave) &&
          (sel_standard_last_scale_type == scale_type))) {

        sel_standard_last_octave = sel_standard_octave;
        sel_standard_last_octave_valid = TRUE;
        sel_standard_last_scale_type = scale_type;

        // Create new menu for optionmenu_noteselect
        menu1 = gtk_menu_new();
    
        menuitem1 = gtk_menu_item_new_with_label("(?)");
        gtk_widget_show(menuitem1);
        gtk_menu_append(GTK_MENU(menu1), GTK_WIDGET(menuitem1));
    
        int2 = sel_standard_octave * 12 - 9;
    
        for (int3 = int2 ; int3 < (int2 + 12) ; int3++) {
            menuitem1 = gtk_menu_item_new();
            /* get the table */
            wid1 = create_note_label(int3);
            gtk_widget_show(menuitem1);
            gtk_widget_show(wid1);
            /* add the table with the sub/super-script note name */
            gtk_container_add(GTK_CONTAINER(menuitem1), wid1);
            gtk_menu_append(GTK_MENU(menu1), GTK_WIDGET(menuitem1));
        }
    
        gtk_widget_show(menu1);
        gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu_noteselect),
                                 GTK_WIDGET(menu1));
        gtk_signal_connect (GTK_OBJECT(menu1), "deactivate",
                            GTK_SIGNAL_FUNC(on_optionmenu_noteselect_menu_deactivate), NULL);
        optionmenu_noteselect_menu = menu1;
    }

    gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu_noteselect),
                                sel_standard_note + 1);

    if (inform_set_note == TRUE)
        sel_standard_inform_set_note();

    return;
}



/*
 * sel_standard_inform_set_note
 *
 * Inform set_note about the new value of optionmenu_noteselect
 *
 */

void sel_standard_inform_set_note()
{
    if (sel_standard_note >= 0) {
        set_note(TRUE, (sel_standard_octave*12) - 9 + sel_standard_note);
    }
    else {
        set_note(FALSE, 0);
    }

    return;
}



/*
 * GTK callback function
 *
 */

void on_button_set_a1_default_clicked (GtkButton *button, gpointer user_data)
{
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_a1freq), 440.0);
    return;
}



/*
 * GTK callback function
 *
 */

void on_button_stop_clicked (GtkButton *button, gpointer user_data)
{
    stop_tuner();
    return;
}



/*
 * GTK callback function
 *
 */

void on_button_start_clicked (GtkButton *button, gpointer user_data)
{
    start_tuner();
    return;
}



/*
 * GTK callback function
 *
 */

gboolean on_window_main_delete_event (GtkWidget *widget, GdkEvent *event,
    gpointer user_data)
{
    exit_program();
    return TRUE;
}



// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// 
gboolean on_drawingarea1_expose_event (GtkWidget *widget,
    GdkEventExpose *event, gpointer user_data)
{
    if (displaystate_is_splash == TRUE)
        gdk_draw_pixmap(drawingarea1->window, GdkGC_black, gdkpixmap1,
                        event->area.x, event->area.y, event->area.x, event->area.y,
                        event->area.width, event->area.height);
    else
        gdk_draw_image(drawingarea1->window, GdkGC_black, gdkimage1,
                       event->area.x, event->area.y, event->area.x, event->area.y,
                       event->area.width, event->area.height);

    return(TRUE);
}



/*
 * GTK callback function
 *
 */

void on_spinbutton_freq_changed (GtkEditable *editable, gpointer user_data)
{
    float float1;

    float1 = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_freq));
    set_frequency(float1);

    // Set note to unset
    if (spinbutton_freq_changed_by_user == TRUE) {
        set_note(FALSE, 0);
    }

    return;
}



/*
 * GTK callback function
 *
 */

void on_spinbutton_a1freq_changed (GtkEditable *editable, gpointer user_data)
{
    set_note(notes_note_set, notes_note);
    return;
}



/*
 * GTK callback function
 *
 */

void on_checkbutton_ministep_freq_toggled (GtkToggleButton *togglebutton,
    gpointer user_data)
{
    gboolean bool1;
    GtkAdjustment *GtkAdjustment1;

    bool1 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_ministep_freq));

    GtkAdjustment1 = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spinbutton_freq));
    if (bool1 == TRUE) {
        GtkAdjustment1->step_increment = 0.01;
        GtkAdjustment1->page_increment = 0.001;
    }
    else {
        GtkAdjustment1->step_increment = 1;
        GtkAdjustment1->page_increment = 10;
    }
    gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(spinbutton_freq), GtkAdjustment1);

    return;
}



/*
 * GTK callback function
 *
 */

void on_checkbutton_ministep_a1freq_toggled (GtkToggleButton *togglebutton,
    gpointer user_data)
{
    gboolean bool1;
    GtkAdjustment *GtkAdjustment1;

    bool1 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_ministep_a1freq));

    GtkAdjustment1 = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spinbutton_a1freq));
    if (bool1 == TRUE) {
        GtkAdjustment1->step_increment=0.01;
        GtkAdjustment1->page_increment=0.001;
    }
    else {
        GtkAdjustment1->step_increment=1;
        GtkAdjustment1->page_increment=10;
    }
    gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(spinbutton_a1freq), GtkAdjustment1);

    return;
}



/*
 * GTK callback function
 *
 */

void on_checkbutton_coarse_tuning_toggled (GtkToggleButton *togglebutton,
    gpointer user_data)
{
    gboolean bool1;

    bool1 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_coarse_tuning));
    tuner_mode_coarse_new=bool1;

    // If tuner is off, redraw.
    check_redo_display();

    return;
}



/**
 * \brief Go up one octave
 *
 */
void on_button_octaveup_clicked (GtkButton *button, gpointer user_data)
{
    sel_standard_octave++;
    sel_standard_update(TRUE);
    return;
}



/**
 * \brief Go down one octave
 *
 */
void on_button_octavedown_clicked (GtkButton *button, gpointer user_data)
{
    sel_standard_octave--;
    sel_standard_update(TRUE);
    return;
}



/*
 * GTK callback function
 *
 */

void on_mmenu_item_settings_activate (GtkMenuItem *menuitem,
                                      gpointer user_data)
{
    if (window_settings == NULL) {
        window_settings = create_window_settings();

        // OSS device conf
        settings_entry_sounddev = lookup_widget(window_settings,
                                                "settings_entry_sounddev");
        gtk_entry_set_text(GTK_ENTRY(settings_entry_sounddev), gstr_sounddev->str);


        // ALSA device conf
        settings_alsa_device = lookup_widget(window_settings,
                                             "settings_alsa_device");
        /* ça plante ici */
        gtk_entry_set_text(GTK_ENTRY(settings_alsa_device), gstr_alsa_device->str);

        // scale radio buttons
        settings_radiobutton_scale_1 = lookup_widget(window_settings,
                                                     "settings_radiobutton_scale_1");
        settings_radiobutton_scale_2 = lookup_widget(window_settings,
                                                     "settings_radiobutton_scale_2");
        if (scale_type == 0)
            gtk_toggle_button_set_active(
                GTK_TOGGLE_BUTTON(settings_radiobutton_scale_1), TRUE);
        if (scale_type == 1)
            gtk_toggle_button_set_active(
                GTK_TOGGLE_BUTTON(settings_radiobutton_scale_2), TRUE);
        gtk_widget_show(window_settings);
    }
    else {
        PRINTF_DEBUG("window_settings != NULL\n");
    }
    return;
}



/**
 * \brief Call exit_program() and exit
 */
void on_mmenu_item_exit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    exit_program();
    return;
}



/*
 * GTK callback function
 *
 */

void on_mmenu_item_sel_guitar_toggled (GtkCheckMenuItem *checkmenuitem,
    gpointer user_data)
{
    if (checkmenuitem->active == TRUE) {
        sel_guitar_show_window();
    }
    else {
        sel_guitar_hide_window();
    }

    return;
}



/*
 * GTK callback function
 *
 */

void on_mmenu_item_about_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    GtkWidget *label_about;
    GString *gstr1=g_string_new("");

    // Message to display in about-dialog
    g_string_sprintf(gstr1,
                     "\n"
                     "- pitchtune -\n"
                     "\n"
                     "Precise Instrument Tweaking\n"
                     "for Crispy Harmony - tuner\n"
                     "\n"
                     "Version %s\n"
                     "License: GPL\n"
                     "\n"
                     "Original author:\n"
                     "Haakon Andre Hjortland\n"
                     "<hahjortland@tande.com>\n"
                     "\n"
                     "Maintainer:\n"
                     "Alexandre Bourget\n"
                     "<alex@bourget.cc>\n"
                     "\n"
                     "Web address: http://pitchtune.sourceforge.net/\n"
                     "Bug tracker: http://www.bourget.cc/pitchtune\n"
                     "\n",
                     VERSION);

    // Create, prepare, and show the about-dialog
    about_dialog=create_about_dialog();
    label_about=lookup_widget(about_dialog, "label_about");
    gtk_label_set_text(GTK_LABEL(label_about), gstr1->str);
    gtk_widget_show(about_dialog);

    // Free the GString
    g_string_free(gstr1, TRUE);

    return;
}



/**
 * \brief Close the settings window and SAVE
 *
 */
void on_settings_button_ok_clicked (GtkButton *button, gpointer user_data)
{
    // Get sound device
    g_string_sprintf(gstr_sounddev, "%s",
                     gtk_entry_get_text(GTK_ENTRY(settings_entry_sounddev)));

    // Get ALSA sound device
    g_string_sprintf(gstr_alsa_device, "%s",
                     gtk_entry_get_text(GTK_ENTRY(settings_alsa_device)));

   // Get scale type
    if (gtk_toggle_button_get_active(
            GTK_TOGGLE_BUTTON(settings_radiobutton_scale_1)) == TRUE)
        scale_type=0;
    if (gtk_toggle_button_get_active(
            GTK_TOGGLE_BUTTON(settings_radiobutton_scale_2)) == TRUE)
        scale_type=1;

    // On-the-fly updates
    sel_standard_update(FALSE); // Update standard note selector
                                // (maybe new scale type)

    // Remove settings window
    gtk_widget_destroy(window_settings);
    return;
}



/*
 * GTK callback function
 *
 */

void on_settings_button_cancel_clicked (GtkButton *button, gpointer user_data)
{
    gtk_widget_destroy(window_settings);
    return;
}



/*
 * GTK callback function
 *
 */

void on_settings_button_browse_sounddev_clicked (GtkButton *button,
    gpointer user_data)
{
    if (fileselection1 == NULL) {
        fileselection1=create_fileselection1();
    
        gtk_signal_connect(
            GTK_OBJECT(GTK_FILE_SELECTION(fileselection1)->ok_button),
            "clicked", GTK_SIGNAL_FUNC(on_fileselection1_ok_clicked), NULL);
    
        gtk_signal_connect(
            GTK_OBJECT(GTK_FILE_SELECTION(fileselection1)->cancel_button),
            "clicked", GTK_SIGNAL_FUNC(on_fileselection1_cancel_clicked), NULL);
    
        gtk_signal_connect(
            GTK_OBJECT(fileselection1),
            "destroy", GTK_SIGNAL_FUNC(on_fileselection1_destroy), NULL);
    
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection1),
                                        gstr_sounddev->str);
    
        gtk_widget_show(fileselection1);
    }
    else {
        PRINTF_DEBUG("fileselection1 != NULL\n");
    }

    return;
}



/**
 * \brief Close the settings window without saving
 *
 */
void on_window_settings_destroy (GtkObject *object, gpointer user_data)
{
    window_settings = NULL;
    return;
}



/*
 * GTK callback function
 *
 */
void on_fileselection1_ok_clicked (GtkFileSelection *selector,
    gpointer user_data)
{
    gtk_entry_set_text(GTK_ENTRY(settings_entry_sounddev),
                       gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection1)));
    gtk_widget_destroy(fileselection1);
    return;
}



/*
 * GTK callback function
 *
 */
void on_fileselection1_cancel_clicked (GtkFileSelection *selector,
    gpointer user_data)
{
    gtk_widget_destroy(fileselection1);
    return;
}



/*
 * GTK callback function
 *
 */
void on_fileselection1_destroy (GtkObject *object, gpointer user_data)
{
    fileselection1 = NULL;
    return;
}



/*
 * GTK callback function
 *
 */

void on_about_dialog_button_ok_clicked (GtkButton *button, gpointer user_data)
{
    gtk_widget_destroy(about_dialog);
    return;
}



/*
 * sel_guitar_set_active
 *
 * Inform main UI about the visibility state of the guitar note selector
 *
 */

void sel_guitar_set_active(gboolean active)
{
gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mmenu_item_sel_guitar),
    active);
return;
}



/*
 * GTK callback function
 *
 */

gboolean on_debug_window_delete_event (GtkWidget *widget, GdkEvent *event,
    gpointer user_data)
{
    gtk_widget_hide(debug_window);
    return(TRUE);
}



/*
 * GTK callback function
 *
 */

void on_mmenu_item_help_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    GtkWidget *label_help;
    GString *gstr1 = g_string_new("");

    // Message to display in help-dialog
    g_string_sprintf(gstr1,
                     "\n"
                     "For help, see:\n"
                     "/usr/share/doc/pitchtune/index.htm\n"
                     "or"
                     "\n"
                     "/usr/local/share/doc/pitchtune/index.htm\n"
                     "(or some place like that)\n"
                     "\n");

    // Create, prepare, and show the about-dialog
    help_dialog = create_help_dialog();
    label_help = lookup_widget(help_dialog, "label_help");
    gtk_label_set_text(GTK_LABEL(label_help), gstr1->str);
    gtk_widget_show(help_dialog);

    // Free the GString
    g_string_free(gstr1, TRUE);
}



/*
 * GTK callback function
 *
 */

void on_help_dialog_button_ok_clicked (GtkButton *button, gpointer user_data)
{
    gtk_widget_destroy(help_dialog);
    return;
}



/*
 * GTK callback function
 *
 */

void on_mmenu_item_small_display_toggled (GtkCheckMenuItem *checkmenuitem,
    gpointer user_data)
{
    display_big = !checkmenuitem->active;

    if (!displaystate_is_splash)
        draw_samplepieces(NULL, TRUE);

    return;
}




/**
 * \brief Toggle the playing of the sine wave
 *
 */
void
on_muting1_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{

    switch(gtk_toggle_button_get_active(button))
        {
        case 1:
            sound_muting(1);
            break;
        case 0:
        default:
            sound_muting(0);
            break;
        }
}


/**
 * \brief Increase one note number, go to the next one.
 */
void
on_button_notedown_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
    set_note(notes_note_set, --notes_note);
    return;
    
}


/**
 * \brief Decrease one note number, go to the next one.
 */
void
on_button_noteup_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    set_note(notes_note_set, ++notes_note);
    return;

}

